@@ -7,9 +7,9 @@
 # under the Creative Commons Attribution 4.0 International License (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 #      http://creativecommons.org/licenses/by/4.0
-# 
+#
 # This file is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
@@ -43,7 +43,7 @@ import configparser
 
 
 
-def computeShape(shape:onnx.onnx_ONNX_REL_1_7_ml_pb2.TensorShapeProto):
+def computeShape(shape):
      outputShape = "("
      reshapeValue = 1
      for dim in shape.dim:
@@ -51,7 +51,7 @@ def computeShape(shape:onnx.onnx_ONNX_REL_1_7_ml_pb2.TensorShapeProto):
           reshapeValue *= dim.dim_value
           outputShape += str(dim.dim_value) +","
        else:
-          outputShape += "1,"  
+          outputShape += "1,"
      tmp = ""
      i =0
      for dim in outputShape:
@@ -63,7 +63,7 @@ def computeShape(shape:onnx.onnx_ONNX_REL_1_7_ml_pb2.TensorShapeProto):
      return outputShape,reshapeValue
 
 
-def computeParam(onnxInput,input_elem_type,param = ""):  
+def computeParam(onnxInput,input_elem_type,param = ""):
      if isinstance(onnxInput,Iterable):
         for elt  in onnxInput:
            if str(elt.type.tensor_type) is not "":
@@ -103,16 +103,16 @@ def modifOnnxPB(onnxInput,nb, prefixName = "res"):
    for i in range(1,nb):
       onnxInput.extend([oneInput])
       onnxInput[i].name = prefixName + str(i)
-      onnxInput[i].type.tensor_type.elem_type = i      
+      onnxInput[i].type.tensor_type.elem_type = i
    return onnxInput
 
 def checkConfiguration(configFile:str):
-# Checking configuration file concistency 
-        
+# Checking configuration file concistency
+
     if not os.path.isfile(configFile):
       print("Configuration file ", configFile," is not found")
       exit()
-      
+
     Config = configparser.ConfigParser()
 
     Config.read(configFile)
@@ -122,7 +122,7 @@ def checkConfiguration(configFile:str):
     errorMsg = f"\033[31mERROR : Bad configuration in " + configFile + " file :\n\n" + "\033[00m"
     
     Ok = True
-    
+
     if 'certificates' in sections and 'proxy' in sections and  'session' in sections:
        try:
           os.environ['CURL_CA_BUNDLE'] = Config.get('certificates', 'CURL_CA_BUNDLE')
@@ -150,17 +150,17 @@ def checkConfiguration(configFile:str):
     else:
        errorMsg = f"\033[31mSections missing in " + configFile +" Configuration file :\033[00m\n 	All [certificates], [proxy] and [session] sections should be defined and filled (see onnx4acumos documentation)"
        Ok = False
-    
+
     if not Ok:
        print(errorMsg)
        exit()
-       
-    return Ok 
-    
+
+    return Ok
+
 
 def run_app_cli():
 
-   # Found where is the setup directory 
+   # Found where is the setup directory
    SETUP_DIR = abspath(dirname(__file__))
 
    # Dump by default (not push)
@@ -177,6 +177,13 @@ def run_app_cli():
    for arg in argv:
        if re.search("ms", arg):
            createMicroService = True
+
+   # deploy ? (not by default)?
+   deploy = False
+   for arg in argv:
+       if re.search("deploy", arg):
+           deploy = True
+
    modelPath = ""
    configFile =""
    modelFileName = "Onnx model should be with .onnx extension or "
@@ -190,20 +197,32 @@ def run_app_cli():
              
    for arg in argv:
        if re.search(".ini", arg):
-          configFile = arg             
-                    
+          configFile = arg
 
-   # Bad command line Help 
-   if modelPath == "" or configFile== "":
-      print("Command line shoud be : onnx4acumos ModelName.onnx configurationFile.ini [-f input.data] [-push [-ms]]")
+
+   # Bad command line Help
+   if modelPath == "" or (configFile== "" and pushSession):
+      print("Command line shoud be : onnx4acumos ModelName.onnx [configurationFile.ini] [-li licenseFile] [-f input.data] [-push configurationFile.ini [-li licenseFile] [-deploy] [-ms]]")
       exit()
 
    # Checking configuration File concistency
-   if not checkConfiguration(configFile):
-      print("Bad configuration file concistency : ",configFile, " (see onnx4acumos documentation to fill it)")     
+   if pushSession:
+      checkConfiguration(configFile)
+
+   # check License File Name is provided
+   licenseFileName = ""
+   found = False
+   for arg in argv:
+       if found:
+          licenseFileName = arg
+          found = False
+       if re.search("-li", arg):
+          found = True
+
+   if licenseFileName != "" and not os.path.isfile(licenseFileName):
+      print("Error : License file ", licenseFileName," is not found")
       exit()
-   
-   
+
    modelFileName = modelPath.split("/")[(len(modelPath.split("/")) - 1)]
 
    #Existence test of the provided Onnx model file
@@ -211,8 +230,8 @@ def run_app_cli():
       print("Model file ", modelPath," is not found")
       exit()
 
-   try:  
-      # Load provided onnx model 
+   try:
+      # Load provided onnx model
       onnx_model = onnx.load(modelPath)
    except:
       print("File ", modelFileName,"is not a ONNX Model")
@@ -248,9 +267,9 @@ def run_app_cli():
    #    <class 'typing.NamedTuple'>}, typing.Dict, typing.List
    #    <class 'numpy.int32'>, <class 'numpy.int64'>, <class 'numpy.float32'>,<class 'numpy.float64'>
 
-   # See file onnx/onnx_ONNX_REL_1_7_ml_pb2.py for protobuf type definitions ( 2 differentes : attribute type or tensor data type)
+   # See file onnx/onnx_ml_pb2.py for protobuf type definitions ( 2 differentes : attribute type or tensor data type)
      # attribute_type = 'UNDEFINED','FLOAT','INT','STR','TENSOR','GRAPH','SPARSE_TENSOR',''FLOATS','INTS','STRS','TENSORS','GRAPHS','SPARSE_TENSORS']
-     # tensor_data_type = ['UNDEFINED','FLOAT','UINT8','INT8','UINT16','INT16','INT32','INT64','STRING','BOOL','FLOAT16','DOUBLE','UINT32','UINT64','COMPLEX64','COMPLEX128', 'BFLOAT16']  
+     # tensor_data_type = ['UNDEFINED','FLOAT','UINT8','INT8','UINT16','INT16','INT32','INT64','STRING','BOOL','FLOAT16','DOUBLE','UINT32','UINT64','COMPLEX64','COMPLEX128', 'BFLOAT16']
 
 
    # Input element type definitions (if tensor type -> numpy N-dimensional array (ndarray) -> List[tensor_type.elem_type])
@@ -260,12 +279,12 @@ def run_app_cli():
 
    for elt in onnxInput:
       try:
-        if str(elt.type.tensor_type) is not "":      
+        if str(elt.type.tensor_type) is not "":
             input_elem_type[elt.name] = ['UNDEFINED','List[np.float32]','List[np.int32]','List[np.int32]','List[np.int32]','List[np.int32]','List[np.int32]','List[np.int64]','str','bool','List[np.float16]','List[np.float64]','List[np.uint32]','List[np.uint64]','List[np.complex64]','List[np.complex128]', 'BFLOAT16']
         else:
-            input_elem_type[elt.name] =  ['UNDEFINED','float','int','str','TENSOR','GRAPH','SPARSE_TENSOR','List[float]','List[int]','List[str]','TENSORS','GRAPHS','SPARSE_TENSORS']     
+            input_elem_type[elt.name] =  ['UNDEFINED','float','int','str','TENSOR','GRAPH','SPARSE_TENSOR','List[float]','List[int]','List[str]','TENSORS','GRAPHS','SPARSE_TENSORS']
       except:
-        input_elem_type[elt.name] =  ['UNDEFINED','float','int','str','TENSOR','GRAPH','SPARSE_TENSOR','List[float]','List[int]','List[str]','TENSORS','GRAPHS','SPARSE_TENSORS'] 
+        input_elem_type[elt.name] =  ['UNDEFINED','float','int','str','TENSOR','GRAPH','SPARSE_TENSOR','List[float]','List[int]','List[str]','TENSORS','GRAPHS','SPARSE_TENSORS']
 
 
 
@@ -277,12 +296,12 @@ def run_app_cli():
 
    for elt in onnxOutput:
       try:
-        if str(elt.type.tensor_type) is not "":      
+        if str(elt.type.tensor_type) is not "":
             output_elem_type[elt.name] = ['UNDEFINED','List[np.float32]','List[np.int32]','List[np.int32]','List[np.int32]','List[np.int32]','List[np.int32]','List[np.int64]','str','bool','List[np.float16]','List[np.float64]','List[np.uint32]','List[np.uint64]','List[np.complex64]','List[np.complex128]', 'BFLOAT16']
         else:
             output_elem_type[elt.name] =  ['UNDEFINED','float','int','str','TENSOR','GRAPH','SPARSE_TENSOR','List[float]','List[int]','List[str]','TENSORS','GRAPHS','SPARSE_TENSORS'] 
       except:
-        output_elem_type[elt.name] =  ['UNDEFINED','float','int','str','TENSOR','GRAPH','SPARSE_TENSOR','List[float]','List[int]','List[str]','TENSORS','GRAPHS','SPARSE_TENSORS'] 
+        output_elem_type[elt.name] =  ['UNDEFINED','float','int','str','TENSOR','GRAPH','SPARSE_TENSOR','List[float]','List[int]','List[str]','TENSORS','GRAPHS','SPARSE_TENSORS']
 
 
    # Compute Input and Output parameters
@@ -315,17 +334,18 @@ def run_app_cli():
        modifOnnxPB(onnxOutput,nb,"OutputSR")
 
 
-   # Taking into account the specificities of the provided Onnx model 
+   # Taking into account the specificities of the provided Onnx model
 
    # Method signature
    templateMethodSignature = "def runOnnxModel"
    newMethodSignature = "def run_" + modelFileName.split(".")[0] +"_OnnxModel(" + onnxInputParam + ")-> MultipleReturn:\n"
 
-   # Configuration File 
-   templateConfigFile = "configFile = " 
+   # Configuration File
+   templateConfigFile = "configFile = "
    newConfigFile = "configFile = \"" + configFile + "\"\n"
-   
-   # reshapedInput 
+
+  
+   # reshapedInput
    templateReshapedInput = "    reshapedInput = np.array"
    newReshapedInput = ""
    for elt in onnxInput:
@@ -334,10 +354,10 @@ def run_app_cli():
              newReshapedInput += "    reshaped"+ elt.name + " = np.array" +"("+ elt.name + ", dtype="+ input_elem_type[elt.name][elt.type.tensor_type.elem_type].split("[")[1].split("]")[0] +").reshape(" +  inputShape + ")\n"
         else:
              newReshapedInput += "    reshaped"+ elt.name + " = " + elt.name + "\n"
-     
 
 
-   # ort_Output 
+
+   # ort_Output
    templateOrt_Output = "    ort_Output = ort_session.run"
    newOrt_Output = templateOrt_Output + "("
 
@@ -349,7 +369,7 @@ def run_app_cli():
           newOrt_Output += "reshaped"+ elt.name + ")\n"
 
 
-   # reshapedOutput  
+   # reshapedOutput
    templateReshapedOutput = "outputData = ort_Output"
    newReshapedOutput = ""
 
@@ -363,11 +383,11 @@ def run_app_cli():
         if re.search("np", outputParam[elt.name]):
            outputShape, outputReshapeValue = computeShape(shape = elt.type.tensor_type.shape)
            newReshapedOutput += "    "+ elt.name + " = ort_Output["+ str(i) + "].reshape(" +  str(outputReshapeValue) + ")\n"
-           newReturn +=  elt.name +" = " + elt.name 
+           newReturn +=  elt.name +" = " + elt.name
         else:
            newReshapedOutput += "    "+ elt.name + " = ort_Output["+ str(i) + "]\n"
            if (outputParam[elt.name] == "List[Elt]"):
-              newReturn +=  elt.name +" = convertDictListToNamedTupleList(" + elt.name + ")" 
+              newReturn +=  elt.name +" = convertDictListToNamedTupleList(" + elt.name + ")"
            else:
               newReturn +=  elt.name +" = " +  elt.name + ""
         if i != maxIter:
@@ -384,7 +404,7 @@ def run_app_cli():
    newMPRC = templateMPRC + "(\"MultipleReturn\", [(\'"
    i = 0
    for elt in onnxOutput:
-         newMPRC+= elt.name + "\', " + outputParam[elt.name] + ")" 
+         newMPRC+= elt.name + "\', " + outputParam[elt.name] + ")"
          if i != maxIter:
             newMPRC+= ", (\'"
          else:
@@ -392,7 +412,7 @@ def run_app_cli():
          i+=1
 
 
-   # Provided model file 
+   # Provided model file
    templateModelFile = "modelFileName ="
    newModelFile = "modelFileName = \"" + modelPath +"\"\n"
 
@@ -422,29 +442,46 @@ def run_app_cli():
 
    if createMicroService:
       newMicroService = "opts = Options(create_microservice=True)\n"
+      if licenseFileName != "":
+         newMicroService = "opts = Options(create_microservice=True,license=\"" + licenseFileName + "\")\n"
    else:
       newMicroService = "opts = Options(create_microservice=False)\n"
+      if licenseFileName != "":
+         newMicroService = "opts = Options(create_microservice=False,license=\"" + licenseFileName + "\")\n"
 
-   # Opening onnxModelOnBoarding template file 
+   if deploy:
+      newMicroService = "opts = Options(create_microservice=True,deploy=True)\n"
+      if licenseFileName != "":
+         newMicroService = "opts = Options(create_microservice=True,deploy=True,license=\"" + licenseFileName + "\")\n"
+
+   # Opening onnxModelOnBoarding template file
    inputFile = open(path_join(SETUP_DIR, 'Templates', 'onnxModelOnBoardingTemplate.py'), "r")
 
-   # model directory creation 
-   dirOnnx = modelFileName.split(".")[0] 
+   # model directory creation
+   dirOnnx = modelFileName.split(".")[0]
 
 
    callMkdir = "mkdir " + dirOnnx
    if not os.path.isdir(dirOnnx):
         subprocess.call(callMkdir,shell=True)
         print("Creation of model onnx directory : ", dirOnnx)
-   
-   # copy configuration file in model onnx directory 
-   cpCall = "cp " + configFile + " "+ dirOnnx 
-   subprocess.call(cpCall,shell=True)
-   # copy onnx model file in model onnx directory    
-   cpCall = "cp " + modelPath + " "+ dirOnnx 
+
+   # copy configuration file in model onnx directory
+   cpCall = "cp " + configFile + " "+ dirOnnx
+   if pushSession or configFile !="":
+      subprocess.call(cpCall,shell=True)
+
+   # copy onnx model file in model onnx directory
+   cpCall = "cp " + modelPath + " "+ dirOnnx
    subprocess.call(cpCall,shell=True)
 
-   # Creation of the new onnxModelOnBoarding file with appropriate features 
+   # copy licence file in model onnx directory
+   cpCall = "cp " + licenseFileName + " "+ dirOnnx
+   if licenseFileName !="":
+      subprocess.call(cpCall,shell=True)
+
+
+   # Creation of the new onnxModelOnBoarding file with appropriate features
    outputFileName = dirOnnx +"/"+ modelFileName.split(".")[0] + '_OnnxModelOnBoarding.py'
    outputFile = open(outputFileName, "w")
 
@@ -492,7 +529,7 @@ def run_app_cli():
        exit()
 
 
-   # Client directory creation 
+   # Client directory creation
 
    dirClient = dirOnnx + "/" + modelFileName.split(".")[0] + "_OnnxClient"
 
@@ -525,7 +562,7 @@ def run_app_cli():
    subprocess.call(cpCall,shell=True)
 
    # pb2 python file creation
-   protocCall = "protoc " + " ./" + dirClient + "/" + modelFileName.split(".")[0] + ".proto" + "  --python_out=." 
+   protocCall = "protoc " + " ./" + dirClient + "/" + modelFileName.split(".")[0] + ".proto" + "  --python_out=."
 
    print("Running ", protocCall)
 
@@ -538,16 +575,16 @@ def run_app_cli():
    subprocess.call(cpCall,shell=True)
 
    # Copy data input file if asked (-f) to Client directory
-   # Data Input File Name 
+   # Data Input File Name
    inputFileName = ""
-   found = False 
+   found = False
    for arg in argv:
        if found:
           inputFileName = arg
           found = False
        if re.search("-f", arg):
           found = True
-        
+
 
    cpCall = "cp " +  inputFileName + "  " + dirClient + "/input"
 
@@ -560,7 +597,7 @@ def run_app_cli():
          print("Copy data input file", inputFileName, "to Client directory : ", dirClient+ "/input")
          subprocess.call(cpCall,shell=True)
 
-   # import protobuf _pb2.py file 
+   # import protobuf _pb2.py file
    templateImportPb = "import model_pb2 as pb"
    newImportPb = "import " +  modelFileName.split(".")[0] + "_pb2" + " as pb\n"
 
@@ -569,7 +606,7 @@ def run_app_cli():
    templateOnLine = "oneLine = inputData"
    newOnLine = "    oneLine = " + onnxInput[0].name + ".reshape(" + str(inputReshapeValue) + ")\n"
 
-   # Input Onnx Extend 
+   # Input Onnx Extend
    # inputOnnx.input.extend(oneLine)
    templateExtend = "inputOnnx.input.extend"
    newExtend = "    inputOnnx." + onnxInput[0].name + ".extend(oneLine)\n"
@@ -580,20 +617,20 @@ def run_app_cli():
    templateReshapeOutput = "output = np.array"
    newReshapeOutput = "    output = np.array(outputOnnx.value).reshape(" + outputShape + ")\n"
 
-   # Client onnx model call 
+   # Client onnx model call
    #   ort_outs = onnxModel
    templateModelCall = "   ort_outs = onnxModel"
-   newModelCall = "   ort_outs = run_" + modelFileName.split(".")[0] +"_OnnxModel(preprocessingData)\n" 
+   newModelCall = "   ort_outs = run_" + modelFileName.split(".")[0] +"_OnnxModel(preprocessingData)\n"
 
-   # modify REST URL 
+   # modify REST URL
    templateRestURL = "restURL ="
    newRestURL = "restURL = \"http://localhost:3330/model/methods/run_" + modelFileName.split(".")[0] + "_OnnxModel\"\n"
 
-   # preprocessing type 
+   # preprocessing type
    templatePreprocessing = "def preprocessing"
    newPreprocessing = "def preprocessing(preProcessingInputFileName: str):\n"
 
-   # postprocessing type 
+   # postprocessing type
    templatePostprocessing = "def postprocessing"
    newPostprocessing = "def postprocessing(postProcessingInput, outputFileName: str)-> bool:\n"
 
@@ -629,11 +666,11 @@ def run_app_cli():
    newModelFile = "modelFileName = \"" + modelFileName +"\"\n"
 
    "**************************************************************************"
-   # Opening Onnx client template file 
+   # Opening Onnx client template file
    inputFile = open(path_join(SETUP_DIR, 'Templates', 'onnxClientTemplate.py'), "r")
 
    # Creation of the onnx client skeleton file with appropriate features
-   print("Creation of the onnx client skeleton file with appropriate features in",dirClient,"directory") 
+   print("Creation of the onnx client skeleton file with appropriate features in",dirClient,"directory")
    outputFileName = dirClient + "/" + modelFileName.split(".")[0] + '_OnnxClientSkeleton.py'
    outputFile = open(outputFileName, "w")
 
@@ -672,7 +709,7 @@ def run_app_cli():
    inputFile.close()
    outputFile.close()
 
- 
+
 if __name__ == '__main__':
     # allow direct run of the cli app for debugging
     run_app_cli()
